@@ -17,6 +17,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import logging
+from typing import List
 
 from gitlabracadabra.objects.object import GitLabracadabraObject
 
@@ -26,13 +27,22 @@ logger = logging.getLogger(__name__)
 
 class GitLabracadabraApplicationSettings(GitLabracadabraObject):
     EXAMPLE_YAML_HEADER = 'application_settings:\n  type: application-settings\n'
-    DOC = [
+    DOC: List[str] = [
+        '# Application settings lifecycle',
+        'gitlab_id',
     ]
     SCHEMA = {
         '$schema': 'http://json-schema.org/draft-04/schema#',
         'title': 'Application settings',
         'type': 'object',
         'properties': {
+            # Standard properties
+            'gitlab_id': {
+                'type': 'string',
+                'description': 'GitLab id',
+                '_example': 'gitlab',
+                '_doc_link': 'action_file.md#gitlab_id',
+            },
             # From https://docs.gitlab.com/ee/api/settings.html#list-of-settings-that-can-be-accessed-via-api-calls
             'admin_notification_email': {
                 'type': 'string',
@@ -760,7 +770,7 @@ class GitLabracadabraApplicationSettings(GitLabracadabraObject):
             'enforce_terms': ['terms'],
             'external_auth_client_cert': ['external_auth_client_key'],
             'external_authorization_service_enabled': ['external_authorization_service_timeout',
-                                                        'external_authorization_service_url'],
+                                                       'external_authorization_service_url'],
             'housekeeping_enabled': ['housekeeping_bitmaps_enabled', 'housekeeping_full_repack_period',
                                      'housekeeping_gc_period', 'housekeeping_incremental_repack_period'],
             'metrics_enabled': ['metrics_host', 'metrics_method_call_threshold', 'metrics_packet_size',
@@ -777,7 +787,7 @@ class GitLabracadabraApplicationSettings(GitLabracadabraObject):
     Return the python-gitlab Gilab object.
     """
     def _object_manager(self):
-        return self._gitlab_manager().settings
+        return self.pygitlab.settings
 
     """"_get()
 
