@@ -1,7 +1,6 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2019-2020 Mathieu Parent <math.parent@gmail.com>
+# Copyright (C) 2019-2021 Mathieu Parent <math.parent@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -20,8 +19,10 @@ import logging
 
 from gitlab.exceptions import GitlabCreateError, GitlabListError
 
+from gitlabracadabra.gitlab.access_levels import access_level
 from gitlabracadabra.mixins.boards import BoardsMixin
 from gitlabracadabra.mixins.groups import GroupsMixin
+from gitlabracadabra.mixins.image_mirrors import ImageMirrorsMixin
 from gitlabracadabra.mixins.labels import LabelsMixin
 from gitlabracadabra.mixins.members import MembersMixin
 from gitlabracadabra.mixins.milestones import MilestonesMixin
@@ -31,18 +32,18 @@ from gitlabracadabra.mixins.rename_branches import RenameBranchesMixin
 from gitlabracadabra.mixins.variables import VariablesMixin
 from gitlabracadabra.mixins.webhooks import WebhooksMixin
 from gitlabracadabra.objects.object import GitLabracadabraObject
-from gitlabracadabra.utils import access_level
 
 
 logger = logging.getLogger(__name__)
 
 
-class GitLabracadabraProject(GitLabracadabraObject, BoardsMixin, GroupsMixin, LabelsMixin, MembersMixin,
-                             MilestonesMixin, MirrorsMixin, PipelineSchedulesMixin, RenameBranchesMixin, VariablesMixin,
-                             WebhooksMixin):
+class GitLabracadabraProject(GitLabracadabraObject, BoardsMixin, GroupsMixin, ImageMirrorsMixin, LabelsMixin,
+                             MembersMixin, MilestonesMixin, MirrorsMixin, PipelineSchedulesMixin, RenameBranchesMixin,
+                             VariablesMixin, WebhooksMixin):
     EXAMPLE_YAML_HEADER = 'mygroup/myproject:\n'
     DOC = [
         '# Project lifecycle',
+        'gitlab_id',
         'create_object',
         'delete_object',
         'archived',
@@ -165,6 +166,9 @@ class GitLabracadabraProject(GitLabracadabraObject, BoardsMixin, GroupsMixin, La
         'unknown_pipeline_schedules',
         'unknown_pipeline_schedule_variables',
 
+        '# Mirroring container images',
+        'image_mirrors',
+
         '# Deprecated',
         'issues_enabled',
         'merge_requests_enabled',
@@ -178,6 +182,12 @@ class GitLabracadabraProject(GitLabracadabraObject, BoardsMixin, GroupsMixin, La
         'type': 'object',
         'properties': {
             # Standard properties
+            'gitlab_id': {
+                'type': 'string',
+                'description': 'GitLab id',
+                '_example': 'gitlab',
+                '_doc_link': 'action_file.md#gitlab_id',
+            },
             'create_object': {
                 'type': 'boolean',
                 'description': 'Create object if it does not exists',
@@ -478,6 +488,10 @@ class GitLabracadabraProject(GitLabracadabraObject, BoardsMixin, GroupsMixin, La
                             'type': 'string',
                             'description': 'Repository URL',
                         },
+                        'auth_id': {
+                            'type': 'string',
+                            'description': 'Section from .python-gitlab.cfg for authentication',
+                        },
                         'direction': {
                             'type': 'string',
                             'description': 'Mirror direction',
@@ -534,6 +548,7 @@ class GitLabracadabraProject(GitLabracadabraObject, BoardsMixin, GroupsMixin, La
                 'uniqueItems': True,
                 '_example': ('\n'
                              '    - url: https://gitlab.com/gitlabracadabra/gitlabracadabra.git\n'
+                             '      # auth_id: gitlab # Section from .python-gitlab.cfg for authentication\n'
                              '      direction: pull # one of pull, push ; only first pull mirror is processed\n'
                              # '      skip_ci: true # defaults to true\n'
                              '      branches: # if you omit this parameter, all branches are mirrored\n'
@@ -556,6 +571,7 @@ class GitLabracadabraProject(GitLabracadabraObject, BoardsMixin, GroupsMixin, La
                              "          to: 'upstream-\\1'\n"
                              # '          skip_ci: true\n'
                              '  builds_access_level: disabled # If you want to prevent triggering pipelines on push'),
+                '_doc_link': 'mirrors.md',
             },
             # From https://docs.gitlab.com/ee/api/projects.html#edit-project
             'mirror': {
@@ -1042,6 +1058,84 @@ class GitLabracadabraProject(GitLabracadabraObject, BoardsMixin, GroupsMixin, La
                 'description': 'What to do with unknown pipeline schedule variables (`warn` by default).',
                 'enum': ['warn', 'delete', 'remove', 'ignore', 'skip'],
             },
+            # GitLabracadabra
+            'image_mirrors': {
+                'type': 'array',
+                'description': 'Container image mirrors',
+                'items': {
+                    'type': 'object',
+                    'properties': {
+                        'from': {
+                            'oneOf': [
+                                {
+                                    'type': 'string',
+                                    'description': 'The source image',
+                                    'pattern': '.+',
+                                },
+                                {
+                                    'type': 'object',
+                                    'properties': {
+                                        'base': {
+                                            'type': 'string',
+                                        },
+                                        'repositories': {
+                                            'type': 'array',
+                                            'items': {
+                                                'type': 'string',
+                                            },
+                                        },
+                                        'tags': {
+                                            'type': 'array',
+                                            'items': {
+                                                'type': 'string',
+                                            },
+                                        },
+                                    },
+                                    'required': ['repositories'],
+                                    'additionalProperties': False,
+                                },
+                            ],
+                        },
+                        'to': {
+                            'oneOf': [
+                                {
+                                    'type': 'string',
+                                    'description': 'The destination image',
+                                },
+                                {
+                                    'type': 'object',
+                                    'properties': {
+                                        'base': {
+                                            'type': 'string',
+                                        },
+                                        'repository': {
+                                            'type': 'string',
+                                        },
+                                        'tag': {
+                                            'type': 'string',
+                                        },
+                                    },
+                                    'additionalProperties': False,
+                                },
+                            ],
+                        },
+                        'semver': {
+                            'type': 'string',
+                            'description': 'Version specification as an NPM range',
+                        },
+                    },
+                    'required': ['from'],
+                    'additionalProperties': False,
+                },
+                '_example': ('\n'
+                             '    # Mirror debian:buster\n'
+                             '    # ... to registry.example.org/mygroup/myproject/library/debian:buster:\n'
+                             "    - from: 'debian:buster'\n"
+                             '    # Overriding destination:\n'
+                             "    - from: 'quay.org/coreos/etcd:v3.4.1'\n"
+                             "      to: 'etcd:v3.4.1' # Default would be coreos/etcd:v3.4.1\n"),
+                '_doc_link': 'image_mirrors.md',
+            },
             # From https://docs.gitlab.com/ee/api/projects.html#edit-project
             'issues_enabled': {
                 'type': 'boolean',
@@ -1200,10 +1294,10 @@ class GitLabracadabraProject(GitLabracadabraObject, BoardsMixin, GroupsMixin, La
             if current_config != target_config_int:
                 if dry_run:
                     logger.info('[%s] NOT Changing protected branch %s access level: %s -> %s (dry-run)',
-                                 self._name, protected_name, current_config, target_config_int)
+                                self._name, protected_name, current_config, target_config_int)
                 else:
                     logger.info('[%s] Changing protected branch %s access level: %s -> %s',
-                                 self._name, protected_name, current_config, target_config_int)
+                                self._name, protected_name, current_config, target_config_int)
                     if 'name' in current_config:
                         self._obj.protectedbranches.delete(protected_name)
                     try:
@@ -1212,7 +1306,7 @@ class GitLabracadabraProject(GitLabracadabraObject, BoardsMixin, GroupsMixin, La
                         if err.response_code != 409:
                             pass
                         logger.warning('[%s] Unable to create protected branch %s: %s',
-                                        self._name, protected_name, err.error_message)
+                                       self._name, protected_name, err.error_message)
         # Remaining protected branches
         if unknown_protected_branches not in ['ignore', 'skip']:
             for protected_name in current_protected_branches:
