@@ -16,25 +16,12 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from unittest import TestCase
-
-from gitlab import Gitlab
-
-import gitlabracadabra.manager
 from gitlabracadabra.objects.application_settings import GitLabracadabraApplicationSettings
-from gitlabracadabra.tests.utils import my_vcr
+from gitlabracadabra.tests import my_vcr
+from gitlabracadabra.tests.case import TestCaseWithManager
 
 
-class TestApplicationSettings(TestCase):
-    def setUp(self):
-        gitlabracadabra.manager._gitlab = Gitlab(  # noqa: S106
-            'http://localhost',
-            private_token='DKkdC5JmKWWZgXZGzg83',
-        )
-
-    def tearDown(self):
-        gitlabracadabra.manager._gitlab = None
-
+class TestApplicationSettings(TestCaseWithManager):
     @my_vcr.use_cassette
     def test_settings(self, cass):
         obj = GitLabracadabraApplicationSettings('memory', 'application_settings', {
